@@ -26,6 +26,8 @@
  ******************************************************************************/
 #pragma once
 
+#include <thrust/detail/config.h>
+
 #if THRUST_DEVICE_COMPILER == THRUST_DEVICE_COMPILER_NVCC
 #include <thrust/detail/cstdint.h>
 #include <thrust/detail/temporary_array.h>
@@ -38,8 +40,9 @@
 #include <thrust/detail/minmax.h>
 #include <thrust/distance.h>
 
-namespace thrust
-{
+#include <cub/util_math.cuh>
+
+THRUST_NAMESPACE_BEGIN
 namespace cuda_cub {
 
 namespace __scan_by_key {
@@ -217,12 +220,12 @@ namespace __scan_by_key {
 
       union TempStorage
       {
-        struct
+        struct ScanStorage
         {
           typename BlockScan::TempStorage              scan;
           typename TilePrefixCallback::TempStorage     prefix;
           typename BlockDiscontinuityKeys::TempStorage discontinuity;
-        };
+        } scan_storage;
 
         typename BlockLoadKeys::TempStorage   load_keys;
         typename BlockLoadValues::TempStorage load_values;
@@ -280,7 +283,7 @@ namespace __scan_by_key {
                 size_value_pair_t &tile_aggregate,
                 thrust::detail::false_type /* is_inclusive */)
       {
-        BlockScan(storage.scan)
+        BlockScan(storage.scan_storage.scan)
             .ExclusiveScan(scan_items, scan_items, scan_op, tile_aggregate);
       }
 
@@ -291,7 +294,7 @@ namespace __scan_by_key {
                 size_value_pair_t &tile_aggregate,
                 thrust::detail::true_type /* is_inclusive */)
       {
-        BlockScan(storage.scan)
+        BlockScan(storage.scan_storage.scan)
             .InclusiveScan(scan_items, scan_items, scan_op, tile_aggregate);
       }
 
@@ -307,7 +310,7 @@ namespace __scan_by_key {
                 TilePrefixCallback &prefix_op,
                 thrust::detail::false_type /* is_incclusive */)
       {
-        BlockScan(storage.scan)
+        BlockScan(storage.scan_storage.scan)
             .ExclusiveScan(scan_items, scan_items, scan_op, prefix_op);
         tile_aggregate = prefix_op.GetBlockAggregate();
       }
@@ -320,7 +323,7 @@ namespace __scan_by_key {
                 TilePrefixCallback &prefix_op,
                 thrust::detail::true_type /* is_inclusive */)
       {
-        BlockScan(storage.scan)
+        BlockScan(storage.scan_storage.scan)
             .InclusiveScan(scan_items, scan_items, scan_op, prefix_op);
         tile_aggregate = prefix_op.GetBlockAggregate();
       }
@@ -423,7 +426,7 @@ namespace __scan_by_key {
         // first tile
         if (tile_idx == 0)
         {
-          BlockDiscontinuityKeys(storage.discontinuity)
+          BlockDiscontinuityKeys(storage.scan_storage.discontinuity)
             .FlagHeads(segment_flags, keys, inequality_op);
 
           // Zip values and segment_flags
@@ -449,7 +452,7 @@ namespace __scan_by_key {
           key_type tile_pred_key = (threadIdx.x == 0)
                                        ? keys_load_it[tile_base - 1]
                                        : key_type();
-          BlockDiscontinuityKeys(storage.discontinuity)
+          BlockDiscontinuityKeys(storage.scan_storage.discontinuity)
               .FlagHeads(segment_flags,
                          keys,
                          inequality_op,
@@ -462,7 +465,7 @@ namespace __scan_by_key {
                                              scan_items);
 
           size_value_pair_t  tile_aggregate;
-          TilePrefixCallback prefix_op(tile_state, storage.prefix, scan_op, tile_idx);
+          TilePrefixCallback prefix_op(tile_state, storage.scan_storage.prefix, scan_op, tile_idx);
           scan_tile(scan_items, tile_aggregate, prefix_op, Inclusive());
         }
 
@@ -670,7 +673,7 @@ namespace __scan_by_key {
     AgentPlan init_plan        = init_agent::get_plan();
 
     int tile_size = scan_by_key_plan.items_per_tile;
-    size_t num_tiles = (num_items + tile_size - 1) / tile_size;
+    size_t num_tiles = cub::DivideAndRoundUp(num_items, tile_size);
 
     size_t vshmem_size = core::vshmem_size(scan_by_key_plan.shared_memory_size,
                                            num_tiles);
@@ -844,14 +847,14 @@ inclusive_scan_by_key(execution_policy<Derived> &policy,
                       ValOutputIt                value_result,
                       BinaryPred                 binary_pred)
 {
-  typedef typename thrust::iterator_traits<ValOutputIt>::value_type value_type;
+  typedef typename thrust::iterator_traits<ValInputIt>::value_type value_type;
   return cuda_cub::inclusive_scan_by_key(policy,
                                          key_first,
                                          key_last,
                                          value_first,
                                          value_result,
                                          binary_pred,
-                                         plus<value_type>());
+                                         thrust::plus<>());
 }
 
 template <class Derived,
@@ -871,7 +874,7 @@ inclusive_scan_by_key(execution_policy<Derived> &policy,
                                          key_last,
                                          value_first,
                                          value_result,
-                                         equal_to<key_type>());
+                                         thrust::equal_to<>());
 }
 
 
@@ -948,7 +951,7 @@ exclusive_scan_by_key(execution_policy<Derived> &policy,
                                          value_result,
                                          init,
                                          binary_pred,
-                                         plus<Init>());
+                                         plus<>());
 }
 
 template <class Derived,
@@ -971,7 +974,7 @@ exclusive_scan_by_key(execution_policy<Derived> &policy,
                                          value_first,
                                          value_result,
                                          init,
-                                         equal_to<key_type>());
+                                         equal_to<>());
 }
 
 
@@ -986,18 +989,18 @@ exclusive_scan_by_key(execution_policy<Derived> &policy,
                       ValInputIt                 value_first,
                       ValOutputIt                value_result)
 {
-  typedef typename iterator_traits<ValOutputIt>::value_type value_type;
+  typedef typename iterator_traits<ValInputIt>::value_type value_type;
   return cuda_cub::exclusive_scan_by_key(policy,
                                          key_first,
                                          key_last,
                                          value_first,
                                          value_result,
-                                         value_type(0));
+                                         value_type{});
 }
 
 
 }    // namespace cuda_cub
-} // end namespace thrust
+THRUST_NAMESPACE_END
 
 #include <thrust/scan.h>
 
