@@ -26,6 +26,7 @@
  ******************************************************************************/
 #pragma once
 
+#include <thrust/detail/config.h>
 
 #if THRUST_DEVICE_COMPILER == THRUST_DEVICE_COMPILER_NVCC
 #include <thrust/system/cuda/config.h>
@@ -42,8 +43,9 @@
 #include <thrust/detail/minmax.h>
 #include <thrust/distance.h>
 
-namespace thrust
-{
+#include <cub/util_math.cuh>
+
+THRUST_NAMESPACE_BEGIN
 
 template <typename DerivedPolicy,
           typename ForwardIterator,
@@ -218,12 +220,12 @@ namespace __unique {
 
       union TempStorage
       {
-        struct
+        struct ScanStorage
         {
           typename BlockScan::TempStorage               scan;
           typename TilePrefixCallback::TempStorage      prefix;
           typename BlockDiscontinuityItems::TempStorage discontinuity;
-        };
+        } scan_storage;
 
         typename BlockLoadItems::TempStorage  load_items;
         shared_items_t shared_items;
@@ -341,13 +343,13 @@ namespace __unique {
 
         if (IS_FIRST_TILE)
         {
-          BlockDiscontinuityItems(temp_storage.discontinuity)
+          BlockDiscontinuityItems(temp_storage.scan_storage.discontinuity)
               .FlagHeads(selection_flags, items_loc, predicate);
         }
         else
         {
           item_type tile_predecessor = items_in[tile_base - 1];
-          BlockDiscontinuityItems(temp_storage.discontinuity)
+          BlockDiscontinuityItems(temp_storage.scan_storage.discontinuity)
               .FlagHeads(selection_flags, items_loc, predicate, tile_predecessor);
         }
 
@@ -367,7 +369,7 @@ namespace __unique {
         Size num_selections_prefix = 0;
         if (IS_FIRST_TILE)
         {
-          BlockScan(temp_storage.scan)
+          BlockScan(temp_storage.scan_storage.scan)
               .ExclusiveSum(selection_flags,
                             selection_idx,
                             num_tile_selections);
@@ -390,10 +392,10 @@ namespace __unique {
         else
         {
           TilePrefixCallback prefix_cb(tile_state,
-                                       temp_storage.prefix,
+                                       temp_storage.scan_storage.prefix,
                                        cub::Sum(),
                                        tile_idx);
-          BlockScan(temp_storage.scan)
+          BlockScan(temp_storage.scan_storage.scan)
               .ExclusiveSum(selection_flags,
                             selection_idx,
                             prefix_cb);
@@ -578,7 +580,7 @@ namespace __unique {
 
 
     int tile_size = unique_plan.items_per_tile;
-    size_t num_tiles = (num_items + tile_size - 1) / tile_size;
+    size_t num_tiles = cub::DivideAndRoundUp(num_items, tile_size);
 
     size_t vshmem_size = core::vshmem_size(unique_plan.shared_memory_size,
                                            num_tiles);
@@ -793,7 +795,7 @@ unique(execution_policy<Derived> &policy,
 }
 
 }    // namespace cuda_cub
-} // end namespace thrust
+THRUST_NAMESPACE_END
 
 //
 #include <thrust/memory.h>
